@@ -9,7 +9,7 @@ IMEC MSU
 """
 
 from M2Crypto import EVP, Err, X509, m2
-from typing import AnyStr, Callable, Optional  # noqa
+from typing import Callable, Optional, Union  # noqa
 
 
 class EngineError(Exception):
@@ -24,8 +24,12 @@ class Engine(object):
 
     m2_engine_free = m2.engine_free
 
-    def __init__(self, id=None, _ptr=None, _pyfree=1):
-        # type: (Optional[bytes], Optional[bytes], int) -> None
+    def __init__(
+        self,
+        id: Optional[bytes] = None,
+        _ptr: Optional[bytes] = None,
+        _pyfree: int = 1,
+    ) -> None:
         """Create new Engine from ENGINE pointer or obtain by id"""
         if not _ptr and not id:
             raise ValueError("No engine id specified")
@@ -36,45 +40,45 @@ class Engine(object):
                 raise ValueError("Unknown engine: %s" % id)
         self._pyfree = _pyfree
 
-    def __del__(self):
-        # type: () -> None
+    def __del__(self) -> None:
         if getattr(self, '_pyfree', 0):
             self.m2_engine_free(self._ptr)
 
-    def init(self):
-        # type: () -> int
+    def init(self) -> int:
         """Obtain a functional reference to the engine.
 
         :return: 0 on error, non-zero on success."""
         return m2.engine_init(self._ptr)
 
-    def finish(self):
-        # type: () -> int
+    def finish(self) -> int:
         """Release a functional and structural reference to the engine."""
         return m2.engine_finish(self._ptr)
 
-    def ctrl_cmd_string(self, cmd, arg, optional=0):
-        # type: (AnyStr, Optional[AnyStr], int) -> None
+    def ctrl_cmd_string(
+        self,
+        cmd: Union[str, bytes],
+        arg: Union[str, bytes, None],
+        optional: int = 0,
+    ) -> None:
         """Call ENGINE_ctrl_cmd_string"""
         if isinstance(cmd, bytes):
             cmd = cmd.decode()
         if arg is not None and isinstance(arg, bytes):
             arg = arg.decode()
-        if not m2.engine_ctrl_cmd_string(self._ptr, cmd, arg, optional):
+        if not m2.engine_ctrl_cmd_string(
+            self._ptr, cmd, arg, optional
+        ):
             raise EngineError(Err.get_error())
 
-    def get_name(self):
-        # type: () -> bytes
+    def get_name(self) -> bytes:
         """Return engine name"""
         return m2.engine_get_name(self._ptr)
 
-    def get_id(self):
-        # type: () -> bytes
+    def get_id(self) -> bytes:
         """Return engine id"""
         return m2.engine_get_id(self._ptr)
 
-    def set_default(self, methods=m2.ENGINE_METHOD_ALL):
-        # type: (int) -> int
+    def set_default(self, methods: int = m2.ENGINE_METHOD_ALL) -> int:
         """
         Use this engine as default for methods specified in argument
 
@@ -82,8 +86,9 @@ class Engine(object):
         """
         return m2.engine_set_default(self._ptr, methods)
 
-    def _engine_load_key(self, func, name, pin=None):
-        # type: (Callable, bytes, Optional[bytes]) -> EVP.PKey
+    def _engine_load_key(
+        self, func: Callable, name: bytes, pin: Optional[bytes] = None
+    ) -> EVP.PKey:
         """Helper function for loading keys"""
         ui = m2.ui_openssl()
         cbd = m2.engine_pkcs11_data_new(pin)
@@ -96,20 +101,25 @@ class Engine(object):
             m2.engine_pkcs11_data_free(cbd)
         return key
 
-    def load_private_key(self, name, pin=None):
-        # type: (bytes, Optional[bytes]) -> X509.X509
+    def load_private_key(
+        self, name: bytes, pin: Optional[bytes] = None
+    ) -> X509.X509:
         """Load private key with engine methods (e.g from smartcard).
-            If pin is not set it will be asked
+        If pin is not set it will be asked
         """
-        return self._engine_load_key(m2.engine_load_private_key, name, pin)
+        return self._engine_load_key(
+            m2.engine_load_private_key, name, pin
+        )
 
-    def load_public_key(self, name, pin=None):
-        # type: (bytes, Optional[bytes]) -> EVP.PKey
+    def load_public_key(
+        self, name: bytes, pin: Optional[bytes] = None
+    ) -> EVP.PKey:
         """Load public key with engine methods (e.g from smartcard)."""
-        return self._engine_load_key(m2.engine_load_public_key, name, pin)
+        return self._engine_load_key(
+            m2.engine_load_public_key, name, pin
+        )
 
-    def load_certificate(self, name):
-        # type: (bytes) -> X509.X509
+    def load_certificate(self, name: bytes) -> X509.X509:
         """Load certificate from engine (e.g from smartcard).
         NOTE: This function may be not implemented by engine!"""
         cptr = m2.engine_load_certificate(self._ptr, name)
@@ -118,8 +128,9 @@ class Engine(object):
         return X509.X509(cptr, _pyfree=1)
 
 
-def load_dynamic_engine(id, sopath):
-    # type: (bytes, AnyStr) -> Engine
+def load_dynamic_engine(
+    id: bytes, sopath: Union[str, bytes]
+) -> Engine:
     """Load and return dymanic engine from sopath and assign id to it"""
     if isinstance(sopath, str):
         sopath = sopath.encode('utf8')
@@ -132,20 +143,17 @@ def load_dynamic_engine(id, sopath):
     return e
 
 
-def load_dynamic():
-    # type: () -> None
+def load_dynamic() -> None:
     """Load dynamic engine"""
     m2.engine_load_dynamic()
 
 
-def load_openssl():
-    # type: () -> None
+def load_openssl() -> None:
     """Load openssl engine"""
     m2.engine_load_openssl()
 
 
-def cleanup():
-    # type: () -> None
+def cleanup() -> None:
     """If you load any engines, you need to clean up after your application
     is finished with the engines."""
     m2.engine_cleanup()
