@@ -14,25 +14,20 @@ class RC4(object):
 
     rc4_free = rc4_free
 
-    def __init__(self, key=None):
-        # type: (Optional[bytes]) -> None
+    def __init__(self, key: Optional[bytes] = None) -> None:
         self.cipher = rc4_new()
         if key:
             rc4_set_key(self.cipher, key)
 
-    def __del__(self):
-        # type: () -> None
+    def __del__(self) -> None:
         if getattr(self, 'cipher', None):
             self.rc4_free(self.cipher)
 
-    def set_key(self, key):
-        # type: (bytes) -> None
+    def set_key(self, key: bytes) -> None:
         rc4_set_key(self.cipher, key)
 
-    def update(self, data):
-        # type: (bytes) -> bytes
+    def update(self, data: bytes) -> bytes:
         return rc4_update(self.cipher, data)
 
-    def final(self):
-        # type: () -> str
+    def final(self) -> str:
         return ''
