@@ -9,8 +9,13 @@ BEEN FINISHED. THE FURTHER WORK WILL BE DONE WHEN THE STATUS OF TWISTED
 IN THE PYTHON 3 (AND ASYNCIO) WORLD WILL BE CLEAR.
 """
 
-__all__ = ['connectSSL', 'connectTCP', 'listenSSL', 'listenTCP',
-           'TLSProtocolWrapper']
+__all__ = [
+    'connectSSL',
+    'connectTCP',
+    'listenSSL',
+    'listenTCP',
+    'TLSProtocolWrapper',
+]
 
 import logging
 
@@ -24,7 +29,7 @@ from M2Crypto.SSL.Checker import Checker, SSLVerificationError
 
 from twisted.internet.interfaces import ITLSTransport
 from twisted.protocols.policies import ProtocolWrapper
-from typing import AnyStr, Callable, Iterable, Optional  # noqa
+from typing import Callable, Iterable, Optional, Union  # noqa
 from zope.interface import implementer
 
 log = logging.getLogger(__name__)
@@ -34,31 +39,46 @@ def _alwaysSucceedsPostConnectionCheck(peerX509, expectedHost):
     return 1
 
 
-def connectSSL(host, port, factory, contextFactory, timeout=30,
-               bindAddress=None,
-               reactor=twisted.internet.reactor,
-               postConnectionCheck=Checker()):
-    # type: (str, int, object, object, int, Optional[str], twisted.internet.reactor, Checker) -> reactor.connectTCP
+def connectSSL(
+    host: str,
+    port: int,
+    factory: object,
+    contextFactory: object,
+    timeout: int = 30,
+    bindAddress: Optional[str] = None,
+    reactor: twisted.internet.reactor = twisted.internet.reactor,
+    postConnectionCheck: Checker = Checker(),
+) -> twisted.internet.reactor.connectTCP:
     """
     A convenience function to start an SSL/TLS connection using Twisted.
 
     See IReactorSSL interface in Twisted.
     """
     wrappingFactory = policies.WrappingFactory(factory)
-    wrappingFactory.protocol = lambda factory, wrappedProtocol: \
-        TLSProtocolWrapper(factory,
-                           wrappedProtocol,
-                           startPassThrough=0,
-                           client=1,
-                           contextFactory=contextFactory,
-                           postConnectionCheck=postConnectionCheck)
-    return reactor.connectTCP(host, port, wrappingFactory, timeout, bindAddress)
+    wrappingFactory.protocol = (
+        lambda factory, wrappedProtocol: TLSProtocolWrapper(
+            factory,
+            wrappedProtocol,
+            startPassThrough=0,
+            client=1,
+            contextFactory=contextFactory,
+            postConnectionCheck=postConnectionCheck,
+        )
+    )
+    return reactor.connectTCP(
+        host, port, wrappingFactory, timeout, bindAddress
+    )
 
 
-def connectTCP(host, port, factory, timeout=30, bindAddress=None,
-               reactor=twisted.internet.reactor,
-               postConnectionCheck=Checker()):
-    # type: (str, int, object, int, Optional[util.AddrType], object, Callable) -> object
+def connectTCP(
+    host: str,
+    port: int,
+    factory: object,
+    timeout: int = 30,
+    bindAddress: Optional[util.AddrType] = None,
+    reactor: object = twisted.internet.reactor,
+    postConnectionCheck: Callable = Checker(),
+) -> object:
     """
     A convenience function to start a TCP connection using Twisted.
 
@@ -67,38 +87,59 @@ def connectTCP(host, port, factory, timeout=30, bindAddress=None,
     See IReactorTCP interface in Twisted.
     """
     wrappingFactory = policies.WrappingFactory(factory)
-    wrappingFactory.protocol = lambda factory, wrappedProtocol: \
-        TLSProtocolWrapper(factory,
-                           wrappedProtocol,
-                           startPassThrough=1,
-                           client=1,
-                           contextFactory=None,
-                           postConnectionCheck=postConnectionCheck)
-    return reactor.connectTCP(host, port, wrappingFactory, timeout, bindAddress)
+    wrappingFactory.protocol = (
+        lambda factory, wrappedProtocol: TLSProtocolWrapper(
+            factory,
+            wrappedProtocol,
+            startPassThrough=1,
+            client=1,
+            contextFactory=None,
+            postConnectionCheck=postConnectionCheck,
+        )
+    )
+    return reactor.connectTCP(
+        host, port, wrappingFactory, timeout, bindAddress
+    )
 
 
-def listenSSL(port, factory, contextFactory, backlog=5, interface='',
-              reactor=twisted.internet.reactor,
-              postConnectionCheck=_alwaysSucceedsPostConnectionCheck):
+def listenSSL(
+    port,
+    factory,
+    contextFactory,
+    backlog=5,
+    interface='',
+    reactor=twisted.internet.reactor,
+    postConnectionCheck=_alwaysSucceedsPostConnectionCheck,
+):
     """
     A convenience function to listen for SSL/TLS connections using Twisted.
 
     See IReactorSSL interface in Twisted.
     """
     wrappingFactory = policies.WrappingFactory(factory)
-    wrappingFactory.protocol = lambda factory, wrappedProtocol: \
-        TLSProtocolWrapper(factory,
-                           wrappedProtocol,
-                           startPassThrough=0,
-                           client=0,
-                           contextFactory=contextFactory,
-                           postConnectionCheck=postConnectionCheck)
-    return reactor.listenTCP(port, wrappingFactory, backlog, interface)
+    wrappingFactory.protocol = (
+        lambda factory, wrappedProtocol: TLSProtocolWrapper(
+            factory,
+            wrappedProtocol,
+            startPassThrough=0,
+            client=0,
+            contextFactory=contextFactory,
+            postConnectionCheck=postConnectionCheck,
+        )
+    )
+    return reactor.listenTCP(
+        port, wrappingFactory, backlog, interface
+    )
 
 
-def listenTCP(port, factory, backlog=5, interface='',
-              reactor=twisted.internet.reactor,
-              postConnectionCheck=None):
+def listenTCP(
+    port,
+    factory,
+    backlog=5,
+    interface='',
+    reactor=twisted.internet.reactor,
+    postConnectionCheck=None,
+):
     """
     A convenience function to listen for TCP connections using Twisted.
 
@@ -107,14 +148,19 @@ def listenTCP(port, factory, backlog=5, interface='',
     See IReactorTCP interface in Twisted.
     """
     wrappingFactory = policies.WrappingFactory(factory)
-    wrappingFactory.protocol = lambda factory, wrappedProtocol: \
-        TLSProtocolWrapper(factory,
-                           wrappedProtocol,
-                           startPassThrough=1,
-                           client=0,
-                           contextFactory=None,
-                           postConnectionCheck=postConnectionCheck)
-    return reactor.listenTCP(port, wrappingFactory, backlog, interface)
+    wrappingFactory.protocol = (
+        lambda factory, wrappedProtocol: TLSProtocolWrapper(
+            factory,
+            wrappedProtocol,
+            startPassThrough=1,
+            client=0,
+            contextFactory=None,
+            postConnectionCheck=postConnectionCheck,
+        )
+    )
+    return reactor.listenTCP(
+        port, wrappingFactory, backlog, interface
+    )
 
 
 class _BioProxy(object):
@@ -125,13 +171,13 @@ class _BioProxy(object):
 
     m2_bio_free_all = m2.bio_free_all
 
-    def __init__(self, bio):
+    def __init__(self, bio: BIO.BIO) -> None:
         self.bio = bio
 
-    def _ptr(self):
+    def _ptr(self) -> BIO.BIO:
         return self.bio
 
-    def __del__(self):
+    def __del__(self) -> Optional[BIO.BIO]:
         if self.bio is not None:
             self.m2_bio_free_all(self.bio)
 
@@ -164,9 +210,15 @@ class TLSProtocolWrapper(ProtocolWrapper):
     which will hook in this class.
     """
 
-    def __init__(self, factory, wrappedProtocol, startPassThrough, client,
-                 contextFactory, postConnectionCheck):
-        # type: (policies.WrappingFactory, object, int, int, object, Checker) -> None
+    def __init__(
+        self,
+        factory: policies.WrappingFactory,
+        wrappedProtocol: object,
+        startPassThrough: int,
+        client: int,
+        contextFactory: object,
+        postConnectionCheck: Checker,
+    ) -> None:
         """
         :param factory:
         :param wrappedProtocol:
@@ -196,13 +248,17 @@ class TLSProtocolWrapper(ProtocolWrapper):
         # factory.wrappedFactory == client/server factory
 
         self.data = b''  # Clear text to encrypt and send
-        self.encrypted = b''  # Encrypted data we need to decrypt and pass on
+        self.encrypted = (
+            b''  # Encrypted data we need to decrypt and pass on
+        )
         self.tlsStarted = 0  # SSL/TLS mode or pass through
         self.checked = 0  # Post connection check done or not
         self.isClient = client
         self.helloDone = 0  # True when hello has been sent
         if postConnectionCheck is None:
-            self.postConnectionCheck = _alwaysSucceedsPostConnectionCheck
+            self.postConnectionCheck = (
+                _alwaysSucceedsPostConnectionCheck
+            )
         else:
             self.postConnectionCheck = postConnectionCheck
 
@@ -255,20 +311,25 @@ class TLSProtocolWrapper(ProtocolWrapper):
         else:
             m2.ssl_set_accept_state(self.ssl._ptr())
 
-        m2.ssl_set_bio(self.ssl._ptr(), self.internalBio, self.internalBio)
-        m2.bio_set_ssl(self.sslBio._ptr(), self.ssl._ptr(), m2.bio_noclose)
+        m2.ssl_set_bio(
+            self.ssl._ptr(), self.internalBio, self.internalBio
+        )
+        m2.bio_set_ssl(
+            self.sslBio._ptr(), self.ssl._ptr(), m2.bio_noclose
+        )
 
         # Need this for writes that are larger than BIO pair buffers
         mode = m2.ssl_get_mode(self.ssl._ptr())
-        m2.ssl_set_mode(self.ssl._ptr(),
-                        mode |
-                        m2.SSL_MODE_ENABLE_PARTIAL_WRITE |
-                        m2.SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER)
+        m2.ssl_set_mode(
+            self.ssl._ptr(),
+            mode
+            | m2.SSL_MODE_ENABLE_PARTIAL_WRITE
+            | m2.SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER,
+        )
 
         self.tlsStarted = 1
 
-    def write(self, data):
-        # type: (bytes) -> None
+    def write(self, data: bytes) -> None:
         if not self.tlsStarted:
             ProtocolWrapper.write(self, data)
             return
@@ -280,11 +341,13 @@ class TLSProtocolWrapper(ProtocolWrapper):
         except BIO.BIOError as e:
             # See http://www.openssl.org/docs/apps/verify.html#DIAGNOSTICS
             # for the error codes returned by SSL_get_verify_result.
-            e.args = (m2.ssl_get_verify_result(self.ssl._ptr()), e.args[0])
+            e.args = (
+                m2.ssl_get_verify_result(self.ssl._ptr()),
+                e.args[0],
+            )
             raise e
 
-    def writeSequence(self, data):
-        # type: (Iterable[bytes]) -> None
+    def writeSequence(self, data: Iterable[bytes]) -> None:
         if not self.tlsStarted:
             ProtocolWrapper.writeSequence(self, b''.join(data))
             return
@@ -300,8 +363,7 @@ class TLSProtocolWrapper(ProtocolWrapper):
         if self.tlsStarted and self.isClient and not self.helloDone:
             self._clientHello()
 
-    def dataReceived(self, data):
-        # type: (bytes) -> None
+    def dataReceived(self, data: bytes) -> None:
         if not self.tlsStarted:
             ProtocolWrapper.dataReceived(self, data)
             return
@@ -324,16 +386,20 @@ class TLSProtocolWrapper(ProtocolWrapper):
         except BIO.BIOError as e:
             # See http://www.openssl.org/docs/apps/verify.html#DIAGNOSTICS
             # for the error codes returned by SSL_get_verify_result.
-            e.args = (m2.ssl_get_verify_result(self.ssl._ptr()), e.args[0])
+            e.args = (
+                m2.ssl_get_verify_result(self.ssl._ptr()),
+                e.args[0],
+            )
             raise e
 
-    def connectionLost(self, reason):
-        # type: (AnyStr) -> None
+    def connectionLost(self, reason: Union[str, bytes]) -> None:
         self.clear()
         ProtocolWrapper.connectionLost(self, reason)
 
     def _check(self):
-        if not self.checked and m2.ssl_is_init_finished(self.ssl._ptr()):
+        if not self.checked and m2.ssl_is_init_finished(
+            self.ssl._ptr()
+        ):
             x509 = m2.ssl_get_peer_cert(self.ssl._ptr())
             if x509 is not None:
                 x509 = X509.X509(x509, 1)
@@ -355,57 +421,55 @@ class TLSProtocolWrapper(ProtocolWrapper):
         except BIO.BIOError as e:
             # See http://www.openssl.org/docs/apps/verify.html#DIAGNOSTICS
             # for the error codes returned by SSL_get_verify_result.
-            e.args = (m2.ssl_get_verify_result(self.ssl._ptr()), e.args[0])
+            e.args = (
+                m2.ssl_get_verify_result(self.ssl._ptr()),
+                e.args[0],
+            )
             raise e
 
     # Optimizations to reduce attribute accesses
 
     @property
-    def _get_wr_guar_ssl(self):
-        # type: () -> Callable[[], int]
+    def _get_wr_guar_ssl(self) -> Callable[[], int]:
         """Return max. length of data can be written to the BIO.
 
         Writes larger than this value will return a value from
         BIO_write() less than the amount requested or if the buffer is
         full request a retry.
         """
-        return partial(m2.bio_ctrl_get_write_guarantee,
-                       self.sslBio._ptr())
+        return partial(
+            m2.bio_ctrl_get_write_guarantee, self.sslBio._ptr()
+        )
 
     @property
-    def _get_wr_guar_net(self):
-        # type: () -> Callable[[], int]
-        return partial(m2.bio_ctrl_get_write_guarantee,
-                       self.networkBio._ptr())
+    def _get_wr_guar_net(self) -> Callable[[], int]:
+        return partial(
+            m2.bio_ctrl_get_write_guarantee, self.networkBio._ptr()
+        )
 
     @property
-    def _shoud_retry_ssl(self):
-        # type: () -> Callable[[], int]
+    def _shoud_retry_ssl(self) -> Callable[[], int]:
         # BIO_should_retry() is true if the call that produced this
         # condition should then be retried at a later time.
         return partial(m2.bio_should_retry, self.sslBio._ptr())
 
     @property
-    def _shoud_retry_net(self):
-        # type: () -> Callable[[], int]
+    def _shoud_retry_net(self) -> Callable[[], int]:
         return partial(m2.bio_should_retry, self.networkBio._ptr())
 
     @property
-    def _ctrl_pend_ssl(self):
-        # type: () -> Callable[[], int]
+    def _ctrl_pend_ssl(self) -> Callable[[], int]:
         # size_t BIO_ctrl_pending(BIO *b);
         # BIO_ctrl_pending() return the number of pending characters in
         # the BIOs read and write buffers.
         return partial(m2.bio_ctrl_pending, self.sslBio._ptr())
 
     @property
-    def _ctrl_pend_net(self):
-        # type: () -> Callable[[], int]
+    def _ctrl_pend_net(self) -> Callable[[], int]:
         return partial(m2.bio_ctrl_pending, self.networkBio._ptr())
 
     @property
-    def _write_ssl(self):
-        # type: () -> Callable[[bytes], int]
+    def _write_ssl(self) -> Callable[[bytes], int]:
         # All these functions return either the amount of data
         # successfully read or written (if the return value is
         # positive) or that no data was successfully read or written
@@ -414,22 +478,20 @@ class TLSProtocolWrapper(ProtocolWrapper):
         return partial(m2.bio_write, self.sslBio._ptr())
 
     @property
-    def _write_net(self):
-        # type: () -> Callable[[bytes], int]
+    def _write_net(self) -> Callable[[bytes], int]:
         return partial(m2.bio_write, self.networkBio._ptr())
 
     @property
-    def _read_ssl(self):
-        # type: () -> Callable[[int], Optional[bytes]]
+    def _read_ssl(self) -> Callable[[int], Optional[bytes]]:
         return partial(m2.bio_read, self.sslBio._ptr())
 
     @property
-    def _read_net(self):
-        # type: () -> Callable[[int], Optional[bytes]]
+    def _read_net(self) -> Callable[[int], Optional[bytes]]:
         return partial(m2.bio_read, self.networkBio._ptr())
 
-    def _encrypt(self, data=b'', clientHello=0):
-        # type: (bytes, int) -> bytes
+    def _encrypt(
+        self, data: bytes = b'', clientHello: int = 0
+    ) -> bytes:
         """
         :param data:
         :param clientHello:
@@ -439,13 +501,18 @@ class TLSProtocolWrapper(ProtocolWrapper):
         self.data += data
 
         while 1:
-            if (self._get_wr_guar_ssl() > 0 and self.data != b'') or clientHello:
+            if (
+                self._get_wr_guar_ssl() > 0 and self.data != b''
+            ) or clientHello:
                 r = self._write_ssl(self.data)
                 if r <= 0:
                     if not self._shoud_retry_ssl():
                         raise IOError(
-                            ('Data left to be written to {}, ' +
-                             'but cannot retry SSL connection!').format(self.sslBio))
+                            (
+                                'Data left to be written to {}, '
+                                + 'but cannot retry SSL connection!'
+                            ).format(self.sslBio)
+                        )
                 else:
                     assert self.checked
                     self.data = self.data[r:]
@@ -453,16 +520,17 @@ class TLSProtocolWrapper(ProtocolWrapper):
             pending = self._ctrl_pend_net()
             if pending:
                 d = self._read_net(pending)
-                if d is not None:  # This is strange, but d can be None
+                if (
+                    d is not None
+                ):  # This is strange, but d can be None
                     encryptedData += d
                 else:
-                    assert(self._shoud_retry_net())
+                    assert self._shoud_retry_net()
             else:
                 break
         return encryptedData
 
-    def _decrypt(self, data=b''):
-        # type: (bytes) -> bytes
+    def _decrypt(self, data: bytes = b'') -> bytes:
         self.encrypted += data
         decryptedData = b''
 
@@ -472,18 +540,23 @@ class TLSProtocolWrapper(ProtocolWrapper):
                 if r <= 0:
                     if not self._shoud_retry_net():
                         raise IOError(
-                            ('Data left to be written to {}, ' +
-                             'but cannot retry SSL connection!').format(self.networkBio))
+                            (
+                                'Data left to be written to {}, '
+                                + 'but cannot retry SSL connection!'
+                            ).format(self.networkBio)
+                        )
                 else:
                     self.encrypted = self.encrypted[r:]
 
             pending = self._ctrl_pend_ssl()
             if pending:
                 d = self._read_ssl(pending)
-                if d is not None:  # This is strange, but d can be None
+                if (
+                    d is not None
+                ):  # This is strange, but d can be None
                     decryptedData += d
                 else:
-                    assert(self._shoud_retry_ssl())
+                    assert self._shoud_retry_ssl()
             else:
                 break
 
