@@ -23,6 +23,7 @@ import gc
 import logging
 import os
 import os.path
+import platform
 import signal
 import socket
 import subprocess
@@ -309,6 +310,7 @@ class HttpslibSSLSNIClientTestCase(BaseSSLClientTestCase):
     def tearDown(self):
         self.ctx.close()
 
+    @unittest.skipIf('DISTROBOX_ENTER_PATH' in os.environ, "Don't run the test on local machine")
     def test_SNI_support(self):
         pid = self.start_server(self.args)
         try:
@@ -329,6 +331,7 @@ class HttpslibSSLSNIClientTestCase(BaseSSLClientTestCase):
             'Hostname in TLS extension: "%s"' % srv_host, out
         )
 
+    @unittest.skipIf('DISTROBOX_ENTER_PATH' in os.environ, "Don't run the test on local machine")
     def test_IP_call(self):
         no_exception = True
         runs_counter = 0
@@ -423,7 +426,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
         finally:
             self.stop_server(pid)
 
-    @unittest.skipIf((util.is_32bit() and util.is_libc_musl()),
+    # the test for musl libc based on the unresolved bug in CPython gh#python/cpython#87414
+    @unittest.skipIf((util.is_32bit() and platform.libc_ver() == ("", "")),
                      "socket.setsockopt() on musl libc fails (srht#mcepl/m2crypto#341)")
     def test_server_simple_timeouts(self):
         pid = self.start_server(self.args)
@@ -1340,14 +1344,6 @@ class FtpsLibTestCase(unittest.TestCase):
         ftpslib.FTP_TLS()
         # XXX need server to test against
 
-
-class SessionTestCase(unittest.TestCase):
-    def test_session_load_bad(self):
-        with self.assertRaises(SSL.SSLError):
-            SSL.Session.load_session('tests/signer.pem')
-
-
-class FtpslibTestCase(unittest.TestCase):
     def test_26_compat(self):
         f = ftpslib.FTP_TLS()
         # 2.6 used to raise AttributeError:
@@ -1359,6 +1355,11 @@ class FtpslibTestCase(unittest.TestCase):
         ):
             f.connect('no-such-host-dfgHJK56789', 990)
 
+class SessionTestCase(unittest.TestCase):
+    def test_session_load_bad(self):
+        with self.assertRaises(SSL.SSLError):
+            SSL.Session.load_session('tests/signer.pem')
+
 
 def suite():
     suite = unittest.TestSuite()
@@ -1371,7 +1372,6 @@ def suite():
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(Urllib2SSLClientTestCase))
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(Urllib2TEChunkedSSLClientTestCase))
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(MiscSSLClientTestCase))
-    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(FtpslibTestCase))
     try:
         import M2Crypto.SSL.TwistedProtocolWrapper as wrapper  # noqa
 
