@@ -17,7 +17,7 @@ import time
 import warnings
 
 from M2Crypto import ASN1, BIO, EVP, RSA, Rand, X509, m2  # noqa
-from M2Crypto.util import is_32bit, is_libc_musl, expectedFailureIf
+from M2Crypto.util import expectedFailureIf
 from tests import unittest
 
 log = logging.getLogger(__name__)
@@ -29,7 +29,7 @@ class X509TestCase(unittest.TestCase):
         pass
 
     def setUp(self):
-        self.expected_hash = 'BA4212E8B55527570828E7F5A0005D17C64BDC4C'
+        self.expected_hash = '46690EBC3862A776CBB37A32CD14ABC5520863BE'
 
     def mkreq(self, bits, ca=0):
         pk = EVP.PKey()
@@ -240,8 +240,10 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(req.as_text(), req4t)
         self.assertEqual(req.as_der(), req4.as_der())
         self.assertEqual(req.get_version(), 0)
-        req.set_version(1)
-        self.assertEqual(req.get_version(), 1)
+
+        if m2.OPENSSL_VERSION_NUMBER < 0x30400000:
+            req.set_version(1)
+            self.assertEqual(req.get_version(), 1)
         req.set_version(0)
         self.assertEqual(req.get_version(), 0)
 
@@ -590,13 +592,13 @@ class X509TestCase(unittest.TestCase):
                          127614157056681299805556476275995414779)
 
     @unittest.skipIf(platform.system() == 'Windows', 'Skip on Windows. TODO')
-    @expectedFailureIf(is_32bit() and not is_libc_musl())
+    @expectedFailureIf(m2.time_t_bits() == 32)
     def test_date_after_2050_working(self):
         cert = X509.load_cert('tests/bad_date_cert.crt')
         self.assertEqual(str(cert.get_not_after()), 'Feb  9 14:57:46 2116 GMT')
 
     @unittest.skipIf(platform.system() == 'Windows', 'Skip on Windows. TODO')
-    @expectedFailureIf(is_32bit() and not is_libc_musl())
+    @expectedFailureIf(m2.time_t_bits() == 32)
     def test_date_reference_counting(self):
         """x509_get_not_before() and x509_get_not_after() return internal
         pointers into X509. As the returned ASN1_TIME objects do not store any
@@ -712,12 +714,12 @@ class X509ExtTestCase(unittest.TestCase):
     def test_ext(self):
         if 0:  # XXX
             # With this leaks 8 bytes:
-            name = "proxyCertInfo"
-            value = "critical,language:Inherit all"
+            name: str = "proxyCertInfo"
+            value: str = "critical,language:Inherit all"
         else:
             # With this there are no leaks:
-            name = "nsComment"
-            value = "Hello"
+            name: str = "nsComment"
+            value: str = "Hello"
 
         ctx = m2.x509v3_set_nconf()
         x509_ext_ptr = m2.x509v3_ext_conf(None, ctx, name, value)
